from bs4 import BeautifulSoup
import pandas as pd
import requests

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(product_name, n_pages):

    flip = "https://www.flipkart.com/search?q="+product_name
    url = requests.get(flip)
    soup = BeautifulSoup(url.text,"lxml")
    
    if n_pages == 0:
        Page = soup.find("div",class_="_2MImiq").find("span", class_="").text
        c = Page.split()
        i=((c[3].replace(',','')))
        print("Enter valid number of Pages between 1 and {}".format(i))
        return scrappi(product_name, n_pages=int(input("Enter a Page Number: ")))
        
    initial_url = str(flip)
    lst_of_urls = []
    for i in range(1, n_pages+1):
        x = initial_url + '&page=' + str(i)
        lst_of_urls.append(x)
    
    def rectangle(soup):
        
        lst = []
        cnt = soup.find_all("div", {"class": "_2kHMtA"})
        for i in range(len(cnt)):
            
            try: name = cnt[i].find("div", {"class": "_4rR01T"}).text  
            except: name = "None"

            try: Price = cnt[i].find("div", {"class":"_30jeq3 _1_WHN1"}).text
            except: Price = "None"

            try:
                Priceo = cnt[i].find("div", {"class": "_3I9_wc _27UcVY"}).text.split()
                oprice = Priceo[0]
            except: oprice = "None"
                
            try: Description = cnt[i].find("li", {"class": "rgWa7D"}).text
            except: Description = "None"

            try: rating=cnt[i].find("div",{"class":"_3LWZlK"}).text
            except: rating="None"
                
            lst.append([name, Price, oprice, Description, rating])

        return lst
    
    def Card_Style(soup):

        lst=[]
        cnt = soup.find_all("div", {"class": "_4ddWXP"})
        for i in range(len(cnt)):
            
            try: name = cnt[i].find("a", {"class": "s1Q9rs"}).text
            except: name = "None"

            try: Price = cnt[i].find("div", {"class": "_30jeq3"}).text
            except: Price = "None"

            try:
                Priceo = cnt[i].find("div", {"class": "_3I9_wc"}).text.split()
                oprice = Priceo[0]

            except: oprice = "None"

            try: Description = cnt[i].find("div", {"class": "_3Djpdu"}).text
            except: Description = "None"
                
            try: rating=cnt[i].find("div",{"class":"_3LWZlK"}).text
            except: rating="None"
                
            lst.append([name, Price, oprice, Description, rating])

        return lst
    
    def OtherStyle(soup):

        lst=[]
        cnt = soup.find_all("div", {"class": "_1xHGtK _373qXS"})
        for i in range(len(cnt)):
            
            try: name = cnt[i].find("div", {"class": "_2WkVRV"}).text
            except: name = "None"

            try: Price = cnt[i].find("div", {"class": "_30jeq3"}).text
            except: Price = "None"

            try:
                Priceo = cnt[i].find("div", {"class": "_3I9_wc"}).text.split()
                oprice = Priceo[0]
            except: oprice = "None"

            try: Description = cnt[i].find("a", {"class": "IRpwTa"}).text
            except: Description = "None"
                
            try: rating=cnt[i].find("div",{"class":"_3LWZlK"}).text
            except: rating="None"

            lst.append([name, Price, oprice, Description, rating])
            
        return lst
    
    def flipkart(soup):
        if len(soup.find_all("div",class_="_4ddWXP"))>=1:
            return Card_Style(soup)
        elif len(soup.find_all("div",class_="_2kHMtA"))>=1:
            return rectangle(soup)
        elif len(soup.find_all("div", {"class": "_1xHGtK _373qXS"}))>=1:
            return OtherStyle(soup)

    x = []
    for i in lst_of_urls:
        url = requests.get(i)
        soup = BeautifulSoup(url.text,"lxml")
        abc = flipkart(soup)
        if abc: 
            for j in abc: x.append(j)

    return pd.DataFrame(x, columns =['Name', 'Price', 'Original Price', 'Description', 'Rating'])


try:
    from webdriver_manager.chrome import ChromeDriverManager
except: raise ImportError("'webdriver-manager' package not installed")
try:
    from selenium.webdriver.common.keys import Keys
    from selenium import webdriver
except: raise ImportError("'selenium' package not installed")
from bs4 import BeautifulSoup
import pandas as pd
import time

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}


def account(insta_handle, n_pages):
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install())
    driver.get("https://www.instagram.com/"+insta_handle+"/")
    
    for _ in range(n_pages):
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)
        
    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    main = soup.find_all('section', {"class": "zwlfE"})
    main2 = soup.find_all('div', {"class": "_2z6nI"})

    for details in main:
        title = details.find('h1', {"class": "rhpdm"})
        info = details.find_all('span', {"class": "g47SY"})
        data = details.find('div', {"class": "-vDIg"}).find_all('span')
        posts = info[0].text
        followers = info[1].text
        following = info[2].text
        if(data): bio = data[0].text
        print("Name: ", title.text)
        print("Number of Posts: ", posts)
        print("Followers: ", followers)
        print("Following: ", following)
        if(data): print("Bio: ", bio) 
        else: print("Bio: None")
        print()
        break

    post_url = []
    for i in main2:
        url = i.find_all('div', {"class": "v1Nh3 kIKUG _bz0w"})
        for x in url:
            u = 'https://www.instagram.com/'+x.a['href']
            post_url.append(u)
    
    def func(a):
        driver.get(a)
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)
        html = driver.page_source
        soup = BeautifulSoup(html, "html.parser")
        main = soup.find_all('div', {"class": "C4VMK"})
        for details in main:
            title = details.find('span', {"class": ""})
            return title.text
            
    caption =[]
    for i in post_url:
        caption.append(func(i))

    driver.close()
    return pd.DataFrame({'Captions': caption})

def hashtag(hashtag, n_posts):
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install())
    hashtag = hashtag.replace('#', '')
    driver.get("https://www.instagram.com/explore/tags/"+hashtag+"/")
    
    for _ in range(int(n_posts//3)):
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)
        
    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    main = soup.find_all('div', {"class": "EZdmt"})

    post_url = []
    for i in main:
        url = i.find_all('div', {"class": "v1Nh3 kIKUG _bz0w"})
        for x in url:
            u = 'https://www.instagram.com'+x.a['href']
            post_url.append(u)
    
    def func(a):
        driver.get(a)
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)
        html = driver.page_source
        soup = BeautifulSoup(html, "html.parser")
        main = soup.find_all('div', {"class": "C4VMK"})
        for details in main:
            title = details.find('span', {"class": ""})
            return title.text
            
    caption =[]
    for i in post_url:
        caption.append(func(i))

    driver.close()
    return pd.DataFrame({'Captions': caption})


try:
    from webdriver_manager.chrome import ChromeDriverManager
except: raise ImportError("'webdriver-manager' package not installed")
try:
    from selenium.webdriver.common.keys import Keys
    from selenium import webdriver
except: raise ImportError("'selenium' package not installed")
from bs4 import BeautifulSoup
import pandas as pd
import time

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(url, n_pages):

    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager().install(), options = chrome_options)
    driver.create_options()
    driver.get(url)

    for _ in range(n_pages):
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)

    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    videos = soup.find_all("div", {"id": "dismissible"})
    lst = []

    for video in videos:
        dictionary = {}
        dictionary['Title'] = video.find("a", {"id": "video-title"}).text
        dictionary['Video_url'] = "https://www.youtube.com/" + video.find("a", {"id": "video-title"})['href']
        meta = video.find("div", {"id": "metadata-line"}).find_all('span')
        dictionary['Views'] = meta[0].text
        dictionary['Days'] = meta[1].text

        lst.append(dictionary)

    return pd.DataFrame(lst)


from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver
from time import sleep
import pandas as pd

########## Stock Analysis ##########
def stock_analysis(stock_code, analysis_type):
    analysis = {
        "earning estimate" : 0, 
        "revenue estimate" : 1,
        "earning history" : 2,
        "EPS trend" : 3,
        "EPS revision" : 4,
        "growth estimate" : 5
    }
    if analysis_type not in analysis.keys(): raise KeyError("Invalid value for 'analysis_type'")
    
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install(), options = chrome_options)
    driver.create_options()
    driver.get("https://in.finance.yahoo.com/quote/"+stock_code+"/analysis?p="+stock_code)
    
    try: tables = driver.find_elements_by_xpath('.//section[@data-test="qsp-analyst"]/table')
    except: raise KeyError("Invalid value for 'stock_code'")

    
    try: df = pd.DataFrame([[data.text for data in row.find_elements_by_xpath('td')] for row in tables[analysis[analysis_type]].find_elements_by_xpath('.//tbody/tr')], columns = [head.text for head in tables[analysis[analysis_type]].find_elements_by_tag_name('th')])
    except: 
        print("Analysis report not Available")
        return None
    return df

########## Stock Profile ##########
def stock_profile(stock_code):
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install(), options = chrome_options)
    driver.create_options()
    try:
        url = "https://in.finance.yahoo.com/quote/"+stock_code+"/profile?p="+stock_code
        driver.get(url)
        sleep(3)

        card = driver.find_element_by_xpath('.//div[@id="Main"]')
        executives = pd.DataFrame([[data.text for data in row.find_elements_by_xpath('td')] for row in card.find_elements_by_xpath('.//tbody/tr')], columns = [i.text for i in card.find_elements_by_tag_name('th')])
        description = card.find_element_by_xpath('.//section/section[2]/p').text
        pro_card = card.find_element_by_xpath('.//div[@data-test="qsp-profile"]')
        profile = {}
        profile["Company Name"] = pro_card.find_element_by_xpath('.//h3').text
        profile["Headquater"] = ', '.join(pro_card.find_element_by_xpath('.//p').text.split('\n')[:3])
        profile["Sector"], profile["Industry"], profile["Employees"] = [i.split(': ')[1] for i in pro_card.find_element_by_xpath('.//p[2]').text.split('\n')]
    except: 
        print("Profile details not Available or the 'stock_code' is Invalid")
        return None, None, None
    return [profile, description, executives]


########## Historical Data ##########
def stock_history(stock_code, stock_range, stock_frequency):
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install(), options = chrome_options)
    driver.create_options()
    
    try:
        url = "https://in.finance.yahoo.com/quote/"+stock_code+"/history?p="+stock_code
        driver.get(url)
        sleep(3)

        card = driver.find_element_by_xpath('//section[@data-test="qsp-historical"]/div[1]')
        time_period = card.find_element_by_xpath('.//div[1]/div[@data-test="dropdown"]')
        time_period.click()
        sleep(3)

        if stock_range == "1d": time_period.find_element_by_xpath('.//button[@data-value="1_D"]').click()
        elif stock_range == "5d":time_period.find_element_by_xpath('.//button[@data-value="5_D"]').click()
        elif stock_range == "3m":time_period.find_element_by_xpath('.//button[@data-value="3_M"]').click()
        elif stock_range == "6m":time_period.find_element_by_xpath('.//button[@data-value="6_M"]').click()
        elif stock_range == "1y":time_period.find_element_by_xpath('.//button[@data-value="1_Y"]').click()
        elif stock_range == "5y":time_period.find_element_by_xpath('.//button[@data-value="5_Y"]').click()
        elif stock_range == "max":time_period.find_element_by_xpath('.//button[@data-value="MAX"]').click()
        else: raise KeyError("Invalid value passed for 'stock_range'")
        sleep(3)

        frequency = card.find_element_by_xpath('.//span[@data-test="historicalFrequency-selected"]')
        frequency.click()
        sleep(3)

        if stock_frequency == "Daily": card.find_element_by_xpath('.//div[@data-value="1d"]').click()
        elif stock_frequency == "Weekly": card.find_element_by_xpath('.//div[@data-value="1wk"]').click()
        elif stock_frequency == "Monthly": card.find_element_by_xpath('.//div[@data-value="1mo"]').click()
        else: raise KeyError("Invalid value passed for 'stock_frequency'")
        sleep(3)

        card.find_element_by_xpath('.//div[1]/button[@data-reactid="25"]').click()
        sleep(3)
    except: 
        print("Historical Data Not Available or the 'stock_code' is Invalid")
        return

    try:
        link = driver.find_elements_by_link_text('Download')[0].get_attribute('href')
        driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install())
        driver.get(link)
        sleep(5)
        print("CSV Downloaded Successfully")
        driver.close()
    except:
        print("CSV file not Available or the 'stock_code' is Invalid")
        return


import pandas as pd
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.keys import Keys
from selenium import webdriver
from bs4 import BeautifulSoup
import time, re

def soundcloud_tracks(track_name, n_pages):
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line = False).install(), options = chrome_options)
    driver.create_options()
    driver.get('https://soundcloud.com/search/sounds?q='+track_name)

    for _ in range(n_pages):
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)

    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    main = soup.find_all("div", {"class": "search"})[0]
    songs = main.find_all('div', {'class': 'sound__content'})
    lst = []
    for song in songs:
        dictionary = {}
        dictionary['Uploader'] = song.find('span', {"class": "soundTitle__usernameText"}).text
        dictionary['Uploader'] = re.sub('[^a-zA-Z]', '', dictionary['Uploader'])
        dictionary['Music Title'] = (song.find('a', {"class": "soundTitle__title sc-link-dark sc-link-secondary"}).text).replace('\n', '')
        dictionary['Time of Upload'] = (song.find('span', {"class": "sc-visuallyhidden"}).text).replace('\n', '')
        dictionary['Plays'] = song.find('span', {"class": "sc-ministats"}).text
        dictionary['Plays'] = re.sub('[^0-9,]', '', dictionary['Plays'])[:-3]
        lst.append(dictionary)

    driver.close()
    return pd.DataFrame(lst)


try:
    from webdriver_manager.chrome import ChromeDriverManager
except: raise ImportError("'webdriver-manager' package not installed")
try:
    from selenium.webdriver.common.keys import Keys
    from selenium import webdriver
except: raise ImportError("'selenium' package not installed")
from bs4 import BeautifulSoup
import pandas as pd
import time

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(n_pages, genre):
    if genre not in  ['national', 'business', 'sports', 'world', 'politics', 'technology', 'startup', 'entertainment', 
        'miscellaneous', 'hatke', 'science', 'automobile']:
        raise ValueError("'genre' value not exists")

    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install(), options = chrome_options)
    driver.create_options()
    driver.get('https://inshorts.com/en/read/'+genre)

    for _ in range(n_pages):
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(3)
        driver.find_element_by_id('load-more-btn').click()
        text_field = driver.find_element_by_id('load-more-btn')
        
    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    main = soup.find_all('div', {"class": "news-card z-depth-1"})
    
    lst = []
    for details in main:
        dictionary={}
        dictionary['Headlines'] = (details.find('a', {"class": "clickable"}).text).replace('\n', '')
        dictionary['Time'] = details.find('span', {"class": "time"}).text
        date = details.find('div', {"class": "news-card-author-time news-card-author-time-in-title"}).find_all('span')
        dictionary['Date'] = date[3].text
        dictionary['News'] = details.find('div', {"itemprop": "articleBody"}).text
        lst.append(dictionary)
        
    return pd.DataFrame(lst)


from bs4 import BeautifulSoup
import requests
import os 

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(data, n_images, img_format, folder_name):
        
    URL = ['https://www.bing.com/images/search?q=', 'https://www.google.com/search?site=&tbm=isch&source=hp&biw=1873&bih=990&q=',
            'https://images.search.yahoo.com/search/images?p=']
    
    def check(folder_name):
        try: 
            os.mkdir(folder_name)
            return folder_name
        except:
            print("Folder Exist with that name!")
            folder_name = input("Enter a new Folder name: \n")
            try: 
                os.mkdir(folder_name)
                return folder_name
            except: return check(folder_name)
            
    folder_name = check(folder_name)

    print('Starting to Download...')

    for i in URL:
        searchurl = i + str(data)
        response = requests.get(searchurl, headers = usr_agent)
        html = response.text
        soup = BeautifulSoup(html, 'html.parser')
        results = soup.findAll('img', limit = n_images)

        if len(results) != 0:
            for i, image in enumerate(results):
                try: image_link = image["data-srcset"]
                except:
                    try: image_link = image["data-src"]
                    except:
                        try: image_link = image["data-fallback-src"]
                        except:
                            try: image_link = image["src"]
                            except: pass
                    try:
                        r = requests.get(image_link).content
                        try: r = str(r, 'utf-8')
                        except UnicodeDecodeError:
                            with open(f"{folder_name}/images{i+1}.{img_format}", "wb+") as f: f.write(r)
                    except: pass
                    
    return 'Successfully Downloaded ' + str(n_images) + ' images'


############## ECommereceScrapper ############## 
class ECommerceScrapper():
    
    """

    ECommerece Scrapper: Helps in scrapping data from E-Comm websites
        1. Alibaba
        2. Flipkart
        3. Snapdeal

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.ECommerceScrapper()

    """

    ############## Alibaba Scrapper ##############
    def alibaba_scrapper(self, product_name, n_pages):

        """

        Alibaba Scrapper: Helps in scrapping Alibaba data ('Name', 'Number of Items', 'Description', 'Ratings').
        return type: DataFrame

        Parameters
        ------------
        product_name: Enter the name of desired product
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion. 
        If n_pages == 0: A prompt will ask you to enter a valid page number and the scrapper will re-run.

        Example
        ---------
        >>>  obj.alibabal_scrapper('product', 3)
        out: Name   Number of Items   Description   Ratings
            abc    440               product a     3.5
            aec    240               product b     4.5

        """

        import alibaba
        return alibaba.scrappi(product_name, n_pages)


    ############## Flipkart Scrapper ############## 
    def flipkart_scrapper(self, product_name, n_pages):

        """

        Flipkart Scrapper: Helps in scrapping Flikart data ('Name', 'Price', 'Original Price', 'Description', 'Rating').
        return type: DataFrame

        Parameters
        ------------
        product_name: Enter the name of the desired product, which you want to scrape the data of
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion. 
        If n_pages == 0: A prompt will ask you to enter a valid page number and the scrapper will re-run. 
        
        Example
        ---------
        >>>  obj.flipkart_scrapper("Product Name", 3) 
        out: Name   Price   Original Price  Description Rating
            abc    ₹340    ₹440            Product     4.2
            aec    ₹140    ₹240            Product     4.7

        """

        import flipkart 
        return flipkart.scrappi(product_name, n_pages)


    ############## Snapdeal Scrapper ##############
    def snapdeal_scrapper(self, product_name, n_pages):

        """

        Snapdeal Scrapper: Helps in scrapping Snapdeal data ('Name', 'Price', 'Original Price', 'Number of Ratings').
        return type: DataFrame

        Parameters
        ------------
        product_name: Enter the name of the desired product, which you want to scrape the data of
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion. 
        If n_pages == 0: A prompt will ask you to enter a valid page number and the scrapper will re-run.
        
        Example
        ---------
        >>>  obj.snapdeal_scrapper('product', 3)
        out: Name   Price   Original Price   Number of Ratings
            abc    ₹340    ₹440             40
            aec    ₹140    ₹240             34

        """

        import snapdeal
        return snapdeal.scrappi(product_name, n_pages)

########################################################################################################################

############## FoodScrapper ##############
class FoodScrapper():

    """

    Food Scrapper: Helps in scrapping data from food delivery websites
        1. Swiggy
        2. Zomato

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.FoodScrapper()

    """

    ############## Swiggy Scrapper ##############
    def swiggy_scrapper(self, city, n_pages):

        """

        Swiggy Scrapper: Helps in scrapping swiggy data ('Name', 'Cuisine', 'Price', 'Rating').
        return type: DataFrame

        Parameters
        ------------
        city: Enter the name of desired city were swiggy delivers
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion. 

        Example
        ---------
        >>>  obj.swiggy_scrapper('city', 3)
        out: Name   Cuisine     Price           Rating
             abc    indian      ₹123 for two    4.5
             xyz    indian      ₹342 for two    4.3

        """

        import swiggy
        return swiggy.scrappi(city, n_pages)


    ############## Zomato Scrapper ##############
    def zomato_scrapper(self, city, n_pages):

        """

        Zomato Scrapper: Helps in scrapping zomato data ("Name", "Cusine", "Price", "Rating", "Delivery Time", "Review counts").
        return type: DataFrame

        Parameters
        ------------
        city: Enter the name of desired city were zomato delivers
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion. 

        Example
        ---------
        >>>  obj.zomato_scrapper('city', 3)
        out: Name   Cuisine     Price           Rating    Delivery Time     Review Count
             abc    indian      ₹123 for two    4.5         45min               3.4K
             xyz    indian      ₹342 for two    4.3         40min               1.2K

        """

        import zomato
        return zomato.scrappi(city, n_pages)

########################################################################################################################

############## Image Scrapper ##############
def image_scrapper(data_name, n_images=10, img_format='jpg', folder_name='images'):

    """

    Image Scrapper: Helps in scrapping images from "Google", "Yahoo", "Bing".
                    Downloads it to the desired folder.

    Parameters
    ------------
    data_name: Enter the name of object/item whose images you want to scrape/download    
        Type: str

    n_images: Enter the number of imsges you want to scrape of download
        Type: int
        Default: 10

    img_format: Enter the format of the image file
        Type: str
        Default: 'jpg'
        Accepted Values: 'jpg', 'jpeg', 'png', 'gif'
    
    folder_name: Enter the path/folder name where you want to download the images
        Type: str
        Default: 'images'

    Note
    ------
    Make sure the data_name is a valid name, and if you enter the directory make sure its a valid one.
    The scrapper will take some time to work. Wait for the images to get scrapped and downloaded, as it scrapes from all three search engines: Google, Yahoo and Bing. 
    
    Feel free to experiment with different image formats.
     
    Example
    ---------
    >>>  image_scarpper('Apple', 100, 'png', 'Apples')
    out: Starting to download...
         Successfully downloaded 100 images 
    
    """

    import image
    return image.scrappi(data_name, n_images, img_format, folder_name)

########################################################################################################################

############## News Scrapper ##############
def news_scrapper(n_pages, genre = str()):

    """

    News Scrapper: Helps in scrapping News (Headlines, Time, Date, News)
    return type: DataFrame

    Parameters
    ------------
    n_pages: Enter the number of pages that it will scrape at a single run.
        Type: int

    genre: Enter the news genre
        Type: str
        Default: str() (None)
        Values accepted: 
        'national', 'business', 'sports', 'world', 'politics', 'technology', 'startup', 'entertainment', 
        'miscellaneous', 'hatke', 'science', 'automobile'

    Note
    ------
    n_pages in a compulsion

    Example
    ---------
    >>>  news_scrapper(3, 'hatke')
    out: Headlines      Time        Date                    News
         New Package    08:19 pm    25 Jun 2021,Sunday      PyScrappy is a new package...
         New Scrapper   08:19 am    25 Jun 2020,Wednesday   PyScrappy is a new Scrapper...

    """
    
    import news
    return news.scrappi(n_pages, genre = genre)

########################################################################################################################

############## Social Media Scrapper ##############
class SocialMediaScrapper():

    """

    Social Media Scrapper: Helps in scrapping data from social media platforms 
        1. Instagram
        2. Twitter
        3. YouTube

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.SocialMediaScrapper()

    """

    ############## Instagram Scrapper ##############
    class InstagramScrapper():

        """

        Instagram Scrapper: Helps in scrapping instagram data (name, posts, followers, following, bio, captions)
            1. Details and post captions based on Insta handle 
            2. Post captions based on #hashtags

        Type: class

        Note
        ------
        Create an object of this class to procced further.

        Example
        ---------
        >>> obj2 = obj.InstagramScrapper()

        """

        ############## Instagram account Scrapper ##############
        def account_scrapper(self, insta_handle, n_pages):

            """

            Instagram account Scrapper: Helps in scrapping instagram data (name, posts, followers, following, bio, captions)
            return type: DataFrame (for captions)

            Parameters
            ------------
            insta_handle: Enter the desired Insta handle/username
                Type: str

            n_pages: Enter the number of pages that you want to scrape
                Type: int

            Note
            ------
            Make sure the Instagram account is public, after certain number of runs, Instagram will ask you for your Instagram ID and PASSWORD, kindly enter it to continue.
            
            Example
            ---------
            >>>  obj2.account_scrapper('Public_account_name', 3)
            out: Name: abc
                Posts: 50
                Followers: 128
                Following: 150
                Bio: Hello World!!

                Captions
                Hello World !!! My first picture.
                Hello World !!! My first program....

            """

            import instagram
            return instagram.account(insta_handle, n_pages)


        ############## Instagram hashtag Scrapper ##############
        def hashtag_scrapper(self, hashtag, n_posts):

            """

            Instagram hashtag Scrapper: Helps in scrapping instagram data (captions)
            return type: DataFrame

            Parameters
            ------------
            hashtag: Enter the desired hashtag
                Type: str

            n_posts: Enter the number of posts that you want to scrape
                Type: int

            Note
            ------
            After certain number of runs, Instagram will ask you for your Instagram ID and PASSWORD, kindly enter it to continue.
            
            Example
            ---------
            >>>  obj2.hashtag_scrapper('#python', 3)
            out: Captions
                Hello World !!! My first picture. #python
                Hello World !!! My first program. #python
                This is scrapping package. #python

            """

            import instagram
            return instagram.hashtag(hashtag, n_posts)


    ############## Twitter Scrapper ##############
    def twitter_scrapper(self, hashtag, n_pages):

        """

        Twitter Scrapper: Helps in scrapping data from Twitter ("Name", "Twitter handle", "Post Time", "Tweet", "Reply Count", "Retweet Count", "Like Count")
        return type: DataFrame

        Parameters
        ------------
        hashtag: Enter the desired hashtag
            Type: str

        n_pages: Enter the number of pages that you want to scrape
            Type: int

        Note
        ------
        Both the arguments are a compulsion

        Example
        ---------
        >>>  obj.twitter_scrapper('#python', 3)
        out: Name   Twitter handle  Post Time   Tweet       Reply Count     Retweet Count   Like Count
             asd    @ksnkj          3:49:36     this is ...     102           230            1.2k
             fsd    @ksdtj          6:49:36     it is a ...     12            30             1k

        """

        import twitter
        return twitter.scrappi(hashtag, n_pages)


    ############## YouTube Scrapper ##############
    def youtube_scrapper(self, video_sec_url, n_pages):

        """

        YouTube Scrapper: Helps in scrapping YouTube data ('Title', 'Video_url', 'Views', 'Days')
        return type: DataFrame

        Parameters
        ------------
        video_sec_url: Enter the desired YouTube URL (only video section)
            Type: str

        n_pages: The number of pages that it will scrape at a single run
            Type: int

        Note
        ------
        Make sure the url is a valid YouTube url, and please enter the url ending with 'videos', i.e urls only from the video sections are acceptable. The scrapping limit is unlimited.
        
        Example
        ---------
        >>>  obj.youtube_scrapper('https://www.youtube.com/user/youtuber_name/videos', 3)
        out: Title       Video_url                                                Views   Days
             My video    https://www.youtube.com/user/youtuber_name/my_video      1.2m    30 days
             My video2   https://www.youtube.com/user/youtuber_name/my_video2     1m      2 weeks
        
        """

        import youtube
        return youtube.scrappi(video_sec_url, n_pages)

########################################################################################################################

############## Song Scrapper ##############
class SongScrapper():
    
    """

    Song Scrapper: Helps in scrapping songs related data
        1. Soundcloud

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.SongScrapper()

    """    

    ############## Soundcloud Scrapper ##############
    def soundcloud_scrapper(self, track_name, n_pages):

        """

        Soundcloud Scrapper: Helps in scrapping data from soundcloud ('Uploader', 'Music Title', 'Time of Upload', 'Plays')
        return type: DataFrame

        Parameters
        ------------
        track_name: Enter the name of desired track/song/music
            Type: str

        n_pages: The number of pages that it will scrape at a single run
            Type: int

        Note
        ------
        Make sure to enter a valid name
        
        Example
        ---------
        >>>  obj.soundcloud_scrapper('music track', 3)
        out: Uploader       Music Title     Time of Upload      Plays
             name1          music           3:34:76             234
             name2          music           5:6:34              445

        """

        import soundcloud
        return soundcloud.soundcloud_tracks(track_name, n_pages)

########################################################################################################################

############## stock Scrapper ##############
class StockScrapper():

    """

    Stock Scrapper: Helps in scrapping stock data
        1. Analysis data of the stock
        2. Historical data of the stock
        3. Profile data of the stock

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.StockScrapper()

    """

    ############## Analysis data scrapper ##############
    def analysis_data_scrapper(self, stock_code, analysis_type):

        """

        Analysis data scrapper: Helps in scrapping the Analytical data of the stock
        return type: DataFrame

        Parameters
        ------------
        stock_code: Enter the desired stock code
            Type: str

        analysis_type: Enter the name of the analysis type of the stock
            Type: str
            Accepted values: "earning estimate", "revenue estimate", "earning history", "EPS trend", "EPS revision", "growth estimate"

        Note
        ------
        Make sure you enter a valid stock code.
        
        Example
        ---------
        >>>  obj.analysis_data_scrapper('STOCK_CODE', 'earning estimates') 

        """

        import stock
        return stock.stock_analysis(stock_code, analysis_type)


    ############## Historical data scrapper ##############
    def historical_data_scrapper(self, stock_code, time_period, frequency):

        """

        Historical data scrapper: Helps in scrapping the Historical data of the stock ('Date', 'Open', 'High', 'Low', 'Close', 'Adjusted Close', 'Volume')
        return type: CSV file

        Parameters
        ------------
        stock_code: Enter the desired stock code
            Type: str

        time_period: Enter the range/time period of the stock
            Type: str
            Accepted values: '1d', '5d', '3m', '6m', '1y', '5y', 'max'

        frequency: Enter the time period interval of the data
            Type: str
            Accepted values: 'Daily', 'Weekly', 'Monthly'

        Note
        ------
        Make sure you enter a valid stock code, time period and time interval 
        
        Example
        ---------
        >>>  obj.historical_data_scrapper('STOCK_CODE', '1y', 'Daily')

        """

        import stock
        return stock.stock_history(stock_code, time_period, frequency)


    ############## Profile details scrapper ##############
    def profile_data_scrapper(self, stock_code):

        """

        Profile data scrapper: Helps in scrapping the Profile data of the stock (profile, description, executives)
        return type: [dict, str, DataFrame]

        Parameters
        ------------
        stock_code: Enter the desired stock code
            Type: str

        Note
        ------
        Make sure you enter a valid stock code.
        Make sure to store the result in three variable as list spreading
        
        Example
        ---------
        >>>  profile, description, executives = obj.profile_data_scrapper('STOCK_CODE')

        """

        import stock
        return stock.stock_profile(stock_code)

########################################################################################################################

############## Wikipedia Scrapper ##############
class WikipediaScrapper():

    """

    Wikipedia Scrapper: Helps in scrapping Wikipedia data
        1. Header
        2. Paragraph
        3. Text

    Type: class

    Note
    ------
    Create an object of this class to procced further.

    Example
    ---------
    >>> obj = PyScrappy.WikipediaScrapper()

    """

    ############## Wikipedia Paragraph Scrapper ##############
    def para_scrapper(self, word):

        """

        Para Scrapper: Helps in scrapping paragraphs from Wikipedia.

        Parameters
        ------------
        word: Enter the desired keyword
            Type: str

        Note
        ------
        Make sure that the info of the word is available in Wikipedia.
        
        Example
        ---------
        >>>  obj.para_scrapper("Python (programming language)")
        out: ['\n',
             "Python is an interpreted high-level general-purpose programming language.", .....]
        
        """

        import wikipedia
        return wikipedia.para(word)

    ############## Wikipedia Header Scrapper ##############
    def header_scrapper(self, word):

        """

        Header Scrapper: Helps in scrapping headers from Wikipedia.

        Parameters
        ------------
        word: Enter the desired keyword
            Type: str

        Note
        ------
        Make sure that the info of the word is available in Wikipedia.
        
        Example
        ---------
        >>>  obj.header_scrapper("Python (programming language)")
        out: ['History',
             'Design philosophy and features', ....]
        
        """

        import wikipedia
        return wikipedia.header(word)

    ############## Wikipedia Text Scrapper ##############
    def text_scrapper(self, word):

        """

        Text Scrapper: Helps in scrapping text from Wikipedia.

        Parameters
        ------------
        word: Enter the desired keyword
            Type: str

        Note
        ------
        Make sure that the info of the word is available in Wikipedia.
        
        Example
        ---------
        >>>  obj.text_scrapper("Python (programming language)")
        out: ' History Python is an interpreted high-level general-purpose programming language..... ' 

        """

        import wikipedia
        return wikipedia.text(word)

########################################################################################################################


from bs4 import BeautifulSoup
import pandas as pd
import requests

def scrappi(city, n_pages):
    lst_of_urls = []
    for i in range(1, n_pages+1): lst_of_urls.append("https://www.swiggy.com/" + city + '?page=' + str(i))

    def swiggy(soup):
        main = soup.find_all('div', {'class': 'nDVxx'})[0]
        lst = []
        for details in main.find_all('div', {'class': '_3XX_A'}):
            dictionary = {}
            dictionary['Name'] = details.find('div', {"class": "nA6kb"}).text
            dictionary['Cuisine'] = details.find('div', {"class": "_1gURR"}).text
            dictionary['Price'] = details.find('div', {"class": "nVWSi"}).text
            dictionary['Rating'] = details.find('div', {"class": "_9uwBC"}).text
            lst.append(dictionary)
        return lst
    
    x = []
    for i in lst_of_urls:
        try: url = requests.get(i)
        except: raise ValueError("Invalid value passed for 'city'")
        soup = BeautifulSoup(url.text, "lxml")
        x.extend(swiggy(soup))

    return pd.DataFrame(x)


from bs4 import BeautifulSoup
import pandas as pd
import requests

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,/;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(product_name, n_pages):

        snap = "https://www.snapdeal.com/search?keyword="+product_name
        url = requests.get(snap)
        soup = BeautifulSoup(url.text,"lxml")
        
        if n_pages == 0:
            print("Enter a valid number of Pages")
            return scrappi(product_name, n_pages=int(input("Enter a Page Number: ")))
            
        initial_url = str(snap)
        lst_of_urls = []
        for i in range(1, n_pages+1):
            x = initial_url + '&page=' + str(i)
            lst_of_urls.append(x)
        
        def Card_Style(soup):

            lst=[]
            cnt = soup.find_all("div", {"class": "product-tuple-description"})
            for i in range(len(cnt)):
                
                try: name = cnt[i].find("p", {"class": "product-title"}).text
                except: name = "None"

                try: Price = cnt[i].find("span", {"class": "lfloat product-price"}).text
                except: Price = "None"

                try: original = cnt[i].find("span", {"class": "lfloat product-desc-price strike"}).text
                except: original = "None"
                    
                try: rating = cnt[i].find("p",{"class":"product-rating-count"}).text
                except: rating = "None"
                    
                lst.append([name, Price, original, rating])

            return lst
        
        def snapdeal(soup):
            if len(soup.find_all("div",class_="product-tuple-description"))>=1:
                return Card_Style(soup)

        x = []
        for i in lst_of_urls:
            url = requests.get(i)
            soup = BeautifulSoup(url.text,"lxml")
            abc = snapdeal(soup)
            for j in abc: x.append(j)

        return pd.DataFrame(x, columns =['Name', 'Price', 'Original Price', 'Number of Ratings'])


from urllib.request import urlopen
from bs4 import BeautifulSoup
import re

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def para(word):
    word=word.replace(' ', '_')
    # Specify url of the web page
    source = urlopen('https://en.wikipedia.org/wiki/'+str(word)).read()
    # Make a soup 
    soup = BeautifulSoup(source,'lxml')

    # Extract the plain text content from paragraphs
    paras = []
    for paragraph in soup.find_all('p'):
        paras.append(str(paragraph.text))
    return paras

def header(word):
    word=word.replace(' ', '_')
    # Specify url of the web page
    source = urlopen('https://en.wikipedia.org/wiki/'+str(word)).read()
    soup = BeautifulSoup(source, "lxml")

    # Extract text from paragraph headers
    heads = []
    for head in soup.find_all('span', attrs={'mw-headline'}):
        heads.append(str(head.text))
    return heads

def text(word):
    word=word.replace(' ', '_')
    # Specify url of the web page
    source = urlopen('https://en.wikipedia.org/wiki/'+str(word)).read()
    soup = BeautifulSoup(source, "lxml")
    
    paras = []
    for paragraph in soup.find_all('p'):
        paras.append(str(paragraph.text))
        
    heads = []
    for head in soup.find_all('span', attrs={'mw-headline'}):
        heads.append(str(head.text))

    # Interleave paragraphs & headers
    text = [val for pair in zip(paras, heads) for val in pair]
    text = ' '.join(text)

    # Drop footnote superscripts in brackets
    text = re.sub(r"\[.*?\]+", '', text)

    # Replace '\n' (a new line) with '' and end the string at $1000.
    text = text.replace('\n', '')[:-11]
    return text


import pandas as pd
from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver
import time

def scrappi(city, n_pages):
    if n_pages == 0: raise ValueError("'n_pages' must be greater than 0")
    city = city.replace(' ', '-')
    driver = webdriver.Chrome(ChromeDriverManager().install())
    driver.get("https://www.zomato.com/"+city+"/restaurants")

    for _ in range(n_pages):
        driver.execute_script('window.scrollBy(0, window.innerHeight);')
        time.sleep(4)

    def zomato(card):
        ls = []
        try: name = card.find_element_by_xpath('.//div/a[2]/div/p').text
        except: name = None
        try: cusine = card.find_element_by_xpath('.//div/a[2]/p').text
        except: cusine = None
        try: rating = card.find_element_by_xpath('.//div/a[2]/div[2]/section').get_attribute('value')
        except: rating = None
        try: price, delivery_time = card.find_element_by_xpath('.//div/a[2]/p[2]').text.split('\n')
        except: price, delivery_time = None, None
        try: reviews_count = card.find_element_by_xpath('.//div/a[2]/div[2]/section/div[2]').text[1:-1]
        except: reviews_count = None
        ls.extend([name, cusine, price, rating, delivery_time, reviews_count])
        return ls

    new_ls = []
    try: cards = driver.find_elements_by_class_name('jumbo-tracker')
    except: raise KeyError("Invalid value for 'city'")
    for card in cards:
        new_ls.append(zomato(card))

    driver.close()
    return pd.DataFrame(new_ls, columns = ["Name", "Cusine", "Price", "Rating", "Delivery Time", "Review counts"])


from setuptools import setup
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="PyScrappy",
    version="0.0.9",
    author="Vedant Tibrewal, Vedaant Singh",
    author_email="mlds93363@gmail.com",
    description="Powerful web scraping tool.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mldsveda/PyScrappy",
    keywords=['PyScrappy', 'Scraping', 'E-Commerce', 'Wikipedia', 'Image Scrapper', 'YouTube', 'Scrapy', 'Twitter', 'Social Media', 'Web Scraping', 'News', 'Stocks', 'Songs', 'Food', 'Instagram'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    py_modules=["PyScrappy", "alibaba", "flipkart", "image", "instagram", "news", "snapdeal", "soundcloud", "stock", "swiggy", "twitter", "wikipedia", "youtube", "zomato"],
    package_dir={"": "src"},
    install_requires=[
        'selenium',
        'webdriver-manager',
        'beautifulsoup4',
        'requests',
        'pandas',
    ],
    packages=setuptools.find_packages(where="src")
)



import pandas as pd
from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver
from time import sleep

def scrappi(hashtag, n_pages):
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.headless = True
    driver = webdriver.Chrome(ChromeDriverManager(print_first_line=False).install(), options = chrome_options)
    driver.create_options()
    driver.get("https://twitter.com/search?q=%23"+hashtag.replace('#', ''))
    sleep(4)

    def twitter(card):
        data_lst = []
        try: name = card.find_element_by_xpath('.//span').text
        except: name = None
        try: twitter_handle = card.find_element_by_xpath('.//span[contains(text(), "@")]').text
        except: twitter_handle = None  
        try: post_time = card.find_element_by_xpath('.//time').get_attribute('datetime')
        except: post_time = None     
        try: tweet = (card.find_element_by_xpath('.//div[2]/div[2]/div[1]').text)+(card.find_element_by_xpath('.//div[2]/div[2]/div[2]').text)
        except: tweet = None     
        try: reply_count = card.find_element_by_xpath('.//div[@data-testid="reply"]').text
        except: reply_count = None     
        try: retweet_count = card.find_element_by_xpath('.//div[@data-testid="retweet"]').text
        except: retweet_count = None     
        try: like_count = card.find_element_by_xpath('.//div[@data-testid="like"]').text
        except: like_count = None 
        
        data_lst.extend([name, twitter_handle, post_time, tweet, reply_count, retweet_count, like_count])
        return data_lst
    
    new_ls = []
    temp_set = set()
    for _ in range(n_pages):
        for card in driver.find_elements_by_xpath('//div[@data-testid="tweet"]'): 
            ls = twitter(card)
            check = ''.join(ls) 
            if check not in temp_set:
                new_ls.append(ls)
                temp_set.add(check)
        driver.execute_script('window.scrollBy(0, window.innerHeight*3);')
        sleep(4)
    
    driver.close()
    return pd.DataFrame(new_ls, columns = ["Name", "Twitter handle", "Post Time", "Tweet", "Reply Count", "Retweet Count", "Like Count"])


from bs4 import BeautifulSoup
import pandas as pd
import requests

usr_agent = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,/;q=0.8',
    'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
    'Accept-Encoding': 'none',
    'Accept-Language': 'en-US,en;q=0.8',
    'Connection': 'keep-alive',
}

def scrappi(product_name, n_pages):
        ali = "https://www.alibaba.com/trade/search?fsb=y&IndexArea=product_en&CatId=&SearchText="+product_name
        url = requests.get(ali)
        soup = BeautifulSoup(url.text,"lxml")
        
        if n_pages == 0:
            print("Enter a valid number of Pages")
            return scrappi(product_name, n_pages=int(input("Enter a Page Number: ")))
            
        initial_url = str(ali)
        lst_of_urls = []
        for i in range(1, n_pages+1):
            x = initial_url + '&page=' + str(i)
            lst_of_urls.append(x)
        
        def Card_Style(soup):

            lst=[]
            cnt = soup.find_all("div", {"class": "m-gallery-product-item-v2"})
            for i in range(len(cnt)):
                
                try: name = cnt[i].find("p", {"class": "elements-title-normal__content"}).text
                except: name = "None"

                try: 
                    Price = cnt[i].find("p", {"class": "elements-offer-price-normal medium"})['title']
                    Price = '$' + str(Price).replace('$', '')
                except: Price = "None"

                try: n_item = cnt[i].find("span", {"class": "element-offer-minorder-normal__value"}).text
                except: n_item = "None"

                try: Description = cnt[i].find("div", {"class": "offer-tag-list"}).text
                except: Description = "None"
                    
                try: rating = cnt[i].find("span",{"class":"seb-supplier-review__score"}).text
                except: rating = "None"
                    
                lst.append([name, Price, n_item, Description, rating])

            return lst
        
        def alibaba(soup):
            if len(soup.find_all("div",class_="m-gallery-product-item-v2"))>=1: return Card_Style(soup)

        x = []
        for i in lst_of_urls:
            url = requests.get(i)
            soup = BeautifulSoup(url.text,"lxml")
            abc = alibaba(soup)
            if abc:
                for j in abc: x.append(j)

        return pd.DataFrame(x, columns =['Name', 'Price', 'Number of Items', 'Description', 'Ratings'])